@@ -254,17 +254,22 @@ use serde::de::DeserializeOwned;
 use std::{
     collections::HashMap,
     convert::Infallible,
+    ops::Deref,
     pin::Pin,
     str::FromStr,
     task::{Context, Poll},
 };
 
+pub mod connect_info;
 pub mod extractor_middleware;
 pub mod rejection;
 
 #[doc(inline)]
 pub use self::extractor_middleware::extractor_middleware;
 
+#[doc(inline)]
+pub use self::connect_info::ConnectInfo;
+
 #[cfg(feature = "multipart")]
 #[cfg_attr(docsrs, doc(cfg(feature = "multipart")))]
 pub mod multipart;
@@ -596,6 +601,14 @@ where
     }
 }
 
+impl<T> Deref for Query<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Extractor that deserializes `application/x-www-form-urlencoded` requests
 /// into some type.
 ///
@@ -667,6 +680,14 @@ where
     }
 }
 
+impl<T> Deref for Form<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Extractor that deserializes request bodies into some type.
 ///
 /// `T` is expected to implement [`serde::Deserialize`].
@@ -731,6 +752,14 @@ where
     }
 }
 
+impl<T> Deref for Json<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 fn has_content_type<B>(
     req: &RequestParts<B>,
     expected_content_type: &str,
@@ -802,13 +831,26 @@ where
             .extensions()
             .ok_or(ExtensionsAlreadyExtracted)?
             .get::<T>()
-            .ok_or(MissingExtension)
+            .ok_or_else(|| {
+                MissingExtension::from_err(format!(
+                    "Extension of type `{}` was not found. Perhaps you forgot to add it?",
+                    std::any::type_name::<T>()
+                ))
+            })
             .map(|x| x.clone())?;
 
         Ok(Extension(value))
     }
 }
 
+impl<T> Deref for Extension<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 #[async_trait]
 impl<B> FromRequest<B> for Bytes
 where
@@ -871,6 +913,8 @@ where
 /// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
 /// # };
 /// ```
+///
+/// [`Stream`]: https://docs.rs/futures/latest/futures/stream/trait.Stream.html
 #[derive(Debug)]
 pub struct BodyStream<B = crate::body::Body>(B);
 
@@ -1071,6 +1115,14 @@ where
     }
 }
 
+impl<T, const N: u64> Deref for ContentLengthLimit<T, N> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Extractor that will get captures from the URL.
 ///
 /// # Example
@@ -1219,6 +1271,14 @@ macro_rules! impl_parse_url {
 
 impl_parse_url!(T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16);
 
+impl<T> Deref for UrlParams<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 fn take_body<B>(req: &mut RequestParts<B>) -> Result<B, BodyAlreadyExtracted> {
     req.take_body().ok_or(BodyAlreadyExtracted)
 }
@@ -1274,6 +1334,16 @@ where
     }
 }
 
+#[cfg(feature = "headers")]
+#[cfg_attr(docsrs, doc(cfg(feature = "headers")))]
+impl<T> Deref for TypedHeader<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Extractor that extracts the raw query string, without parsing it.
 ///
 /// # Example
