@@ -3,6 +3,7 @@
 use crate::{
     body::{box_body, BoxBody},
     buffer::MpscBuffer,
+    extract::connect_info::{Connected, IntoMakeServiceWithConnectInfo},
     response::IntoResponse,
     util::ByteStr,
 };
@@ -84,7 +85,6 @@ pub struct Route<S, F> {
 }
 
 /// Trait for building routers.
-// TODO(david): this name isn't great
 #[async_trait]
 pub trait RoutingDsl: crate::sealed::Sealed + Sized {
     /// Add another route to the router.
@@ -267,6 +267,93 @@ pub trait RoutingDsl: crate::sealed::Sealed + Sized {
     {
         tower::make::Shared::new(self)
     }
+
+    /// Convert this router into a [`MakeService`], that will store `C`'s
+    /// associated `ConnectInfo` in a request extension such that [`ConnectInfo`]
+    /// can extract it.
+    ///
+    /// This enables extracting things like the client's remote address.
+    ///
+    /// Extracting [`std::net::SocketAddr`] is supported out of the box:
+    ///
+    /// ```
+    /// use axum::{prelude::*, extract::ConnectInfo};
+    /// use std::net::SocketAddr;
+    ///
+    /// let app = route("/", get(handler));
+    ///
+    /// async fn handler(ConnectInfo(addr): ConnectInfo<SocketAddr>) -> String {
+    ///     format!("Hello {}", addr)
+    /// }
+    ///
+    /// # async {
+    /// hyper::Server::bind(&"0.0.0.0:3000".parse().unwrap())
+    ///     .serve(
+    ///         app.into_make_service_with_connect_info::<SocketAddr, _>()
+    ///     )
+    ///     .await
+    ///     .expect("server failed");
+    /// # };
+    /// ```
+    ///
+    /// You can implement custom a [`Connected`] like so:
+    ///
+    /// ```
+    /// use axum::{
+    ///     prelude::*,
+    ///     extract::connect_info::{ConnectInfo, Connected},
+    /// };
+    /// use hyper::server::conn::AddrStream;
+    ///
+    /// let app = route("/", get(handler));
+    ///
+    /// async fn handler(
+    ///     ConnectInfo(my_connect_info): ConnectInfo<MyConnectInfo>,
+    /// ) -> String {
+    ///     format!("Hello {:?}", my_connect_info)
+    /// }
+    ///
+    /// #[derive(Clone, Debug)]
+    /// struct MyConnectInfo {
+    ///     // ...
+    /// }
+    ///
+    /// impl Connected<&AddrStream> for MyConnectInfo {
+    ///     type ConnectInfo = MyConnectInfo;
+    ///
+    ///     fn connect_info(target: &AddrStream) -> Self::ConnectInfo {
+    ///         MyConnectInfo {
+    ///             // ...
+    ///         }
+    ///     }
+    /// }
+    ///
+    /// # async {
+    /// hyper::Server::bind(&"0.0.0.0:3000".parse().unwrap())
+    ///     .serve(
+    ///         app.into_make_service_with_connect_info::<MyConnectInfo, _>()
+    ///     )
+    ///     .await
+    ///     .expect("server failed");
+    /// # };
+    /// ```
+    ///
+    /// See the [unix domain socket example][uds] for an example of how to use
+    /// this to collect UDS connection info.
+    ///
+    /// [`MakeService`]: tower::make::MakeService
+    /// [`Connected`]: crate::extract::connect_info::Connected
+    /// [`ConnectInfo`]: crate::extract::connect_info::ConnectInfo
+    /// [uds]: https://github.com/tokio-rs/axum/blob/main/examples/unix_domain_socket.rs
+    fn into_make_service_with_connect_info<C, Target>(
+        self,
+    ) -> IntoMakeServiceWithConnectInfo<Self, C>
+    where
+        Self: Clone,
+        C: Connected<Target>,
+    {
+        IntoMakeServiceWithConnectInfo::new(self)
+    }
 }
 
 impl<S, F> RoutingDsl for Route<S, F> {}
@@ -364,21 +451,38 @@ fn insert_url_params<B>(req: &mut Request<B>, params: Vec<(String, String)>) {
     }
 }
 
-/// A [`Service`] that responds with `404 Not Found` to all requests.
+/// A [`Service`] that responds with `404 Not Found` or `405 Method not allowed`
+/// to all requests.
 ///
 /// This is used as the bottom service in a router stack. You shouldn't have to
 /// use to manually.
-pub struct EmptyRouter<E = Infallible>(PhantomData<fn() -> E>);
+pub struct EmptyRouter<E = Infallible> {
+    status: StatusCode,
+    _marker: PhantomData<fn() -> E>,
+}
 
 impl<E> EmptyRouter<E> {
-    pub(crate) fn new() -> Self {
-        Self(PhantomData)
+    pub(crate) fn not_found() -> Self {
+        Self {
+            status: StatusCode::NOT_FOUND,
+            _marker: PhantomData,
+        }
+    }
+
+    pub(crate) fn method_not_allowed() -> Self {
+        Self {
+            status: StatusCode::METHOD_NOT_ALLOWED,
+            _marker: PhantomData,
+        }
     }
 }
 
 impl<E> Clone for EmptyRouter<E> {
     fn clone(&self) -> Self {
-        Self(PhantomData)
+        Self {
+            status: self.status,
+            _marker: PhantomData,
+        }
     }
 }
 
@@ -405,7 +509,7 @@ impl<B, E> Service<Request<B>> for EmptyRouter<E> {
 
     fn call(&mut self, _req: Request<B>) -> Self::Future {
         let mut res = Response::new(crate::body::empty());
-        *res.status_mut() = StatusCode::NOT_FOUND;
+        *res.status_mut() = self.status;
         EmptyRouterFuture(future::ok(res))
     }
 }
@@ -806,7 +910,7 @@ where
     Nested {
         pattern: PathPattern::new(description),
         svc,
-        fallback: EmptyRouter::new(),
+        fallback: EmptyRouter::not_found(),
     }
 }
 
